import os
import random
import pathlib
import openai

# Uncomment when using pytest and uploading the package to PyPi
from pyrizz.pickuplines import pickuplines
from pyrizz.templates import templates

# Uncomment when testing the __main__.py file locally
# from pickuplines import pickuplines
# from templates import templates

PROJECT_ROOT = f"{pathlib.Path(__file__).parent.resolve()}/"

def get_lines(category='all'): 
    if category not in pickuplines:
        return ("Category does not exist!")

    else: 
        return pickuplines[category]

def get_random_line(): 
    allpickuplines = get_lines('all')
    return random.choice(allpickuplines)

def get_random_category_line(category='all'):
    category_pickupline = get_lines(category)
    return random.choice(category_pickupline)

def init_openai(key):
    try:
        openai.api_key = key
        openai.Model.list()
        print("\nSuccessful. You can use AI functionality now.\n")
        return openai
    except openai.error.AuthenticationError as err:
        print("\nAuthentication failed. Try entering a different API key or call on pyrizz.init_openai(your_key).\n")
        return err

def get_ai_line(category, client) -> str:
    try:
        if (category != "" and len(category) <= 50):
            response = client.ChatCompletion.create(
                model = "gpt-3.5-turbo",
                messages =
                    [{"role": "user", "content": f"I need a {category} pick-up line."},]
            )

            message = response.choices[0]['message']
            ai_line = "{}".format(message['content'])
            return ai_line
        
        elif (category != "" and len(category) > 50):
            return "Please specify a category that is less than 50 characters."
        
        else:
            return "Please specify a category."
            
    except Exception as err:
        return str(err)

def rate_line(pickup_line, client) -> str:
    try:
        if (pickup_line != ""):
            response = client.ChatCompletion.create(
                model = "gpt-3.5-turbo",
                messages =
                    [{"role": "user", "content": f"Rate this pickup line out of 10 (whole numbers only): {pickup_line} In your response, STRICTLY follow the format of (nothing else): rating/10 - snazzy comment."},]
            )
            message = response.choices[0]['message']
            ai_rating_response = "{}".format(message['content'])
            return ai_rating_response
        
        else:
            return "No pickup line? You gotta use our other features before you come here buddy."
        
    except Exception as err:
        return str(err)

def load_ascii_art(filename):
    with open(filename, 'r', encoding='utf-8') as file:
        content = file.read()
    ascii_art_pieces = content.split('[End]')
    cleaned_ascii_art_pieces = []
    for piece in ascii_art_pieces:
        if '[Start]' in piece:
            start_index = piece.find('[Start]') + len('[Start]')
            art_piece = piece[start_index:]
            cleaned_ascii_art_pieces.append(art_piece)
    return cleaned_ascii_art_pieces

def create_line(template_number, words):
    if not (0 <= template_number < len(templates)):
        return None, "Template number out of range. Please choose between 0 and {}.".format(len(templates) - 1)

    template_to_use = templates[template_number]
    placeholders_count = template_to_use.count("{}")
    if placeholders_count != len(words):
        return None, "Incorrect number of words provided for the placeholders. Expected {}, got {}.".format(placeholders_count, len(words))

    try:
        user_line = template_to_use.format(*words)
    except IndexError as e:
        return None, f"Error in formatting: {str(e)}"

    if is_line_valid(user_line):
        ascii_art_pieces = load_ascii_art(PROJECT_ROOT + 'data/ascii_art.txt')
        art = random.choice(ascii_art_pieces)
        user_line_with_art = f"{art}\n\n{user_line}"
        return user_line_with_art
    else:
        return None, "Your line doesn't pass our checks. Sorry!"

def get_user_input_for_line():
    print("Choose a template number (0-{}):".format(len(templates) - 1))
    try:
        template_number = int(input("> "))  
    except ValueError:
        print("Invalid input. Please enter an integer value.")
        return None, None

    if template_number not in range(len(templates)):
        print("Invalid template number. Please choose a number between 0 and {}.".format(len(templates) - 1))
        return None, None

    template_to_show = templates[template_number]
    placeholders_count = template_to_show.count("{}")

    print("Fill in the blanks for the following template:")
    print(template_to_show.format(*(['______'] * placeholders_count)))

    print(f"Enter {placeholders_count} word(s) separated by commas to fill into the template:")
    words_input = input("> ")
    words = [word.strip() for word in words_input.split(',')]

    if len(words) != placeholders_count:
        print(f"Incorrect number of words. You need to provide exactly {placeholders_count} word(s).")
        return None, None

    return template_number, words

def is_line_valid(user_line):
    if len(user_line) > 140:
        print("Your pick-up line is too long.")
        return False
    
    return True

def list_templates():
    return templates



templates = [
    "Do you have a {}? Because whenever I look at you, everyone else disappears.",
    "Are you a {}? Because every time I think of you, I smile.",
    "Do you have a map? I keep getting lost in your {}.",
    "If beauty were time, you'd be an {}.",
    "I'm not a photographer, but I can picture us together in a {}.",
    "Can I follow you home? Cause my parents always told me to follow my {}.",
    "Is your name Google? Because you have everything I've been searching for in the {}.",
    "Are you made of copper and tellurium? Because you're {}.",
    "Do you like Star Wars? Because Yoda only one for me in the {}.",
    "If you were a {} on a McDonald's menu, you'd be a McGorgeous.",
    "Do you have a Band-Aid? Because I just scraped my knee falling for {}.",
    "Is it hot in here or is it just the {} we have together?",
    "If you were a vegetable, you'd be a {}.",
    "I must be a {} because I am attracted to your magnetic field.",
    "Is there an airport nearby or is that just my heart taking {}?",
    "You must be a magician because whenever I look at you, everyone else {}.",
    "If I could rearrange the alphabet, I'd put {} and {} together.",
    "Are you a {}? Because you're so fine you blow my mind.",
    "Do you have a pencil? Cause I want to erase your past and write our {}.",
    "Do you know what my shirt is made of? It's made of boyfriend {}.",
    "Are you a {}? Because you just brightened my day with a single smile.",
    "I'm not a genie, but I can make your {} come true.",
    "If you were a {}, you'd be a sweet one.",
    "Was that an earthquake, or did you just rock my {}?",
    "Do you have a name, or can I call you {}?",
    "If we were at home, cuddling on a rainy Sunday morning, what would we have for {}? Pancakes, bacon, or {}?",
    "Excuse me, but I think you dropped something: MY {}.",
    "If you were a fruit, you'd be a {}apple.",
    "Are you a {}? Because every time I look at you, I feel like I'm on cloud nine.",
    "Do you play soccer? Because you're a {} in the field.",
    "If you were words on a page, you'd be the {} print.",
    "Did we just share an {}? Because I feel like I've known you forever.",
    "I'm writing a term paper on the finer things in life. Can I interview you over {}?",
    "I'd never play hide and seek with you because someone like you is impossible to {}.",
    "Can I take a picture of you, so I can show Santa what I want for {}?",
    "Do you have a sunburn, or are you always this {}?",
    "If you were a {} in the sea, I'd swim all the oceans to find you.",
    "You must be the square root of -1 because you can't be {}.",
    "Is it okay if I follow you home? Because my parents always told me to follow my {}.",
    "If you were a {}, you'd be an extraordinary one."
]



dev_lines = [
    "Is it okay if I walk you home? My heart seems to follow wherever you go.\n",
    "If beauty were a moment, you'd be an eternity.\n",
    "Did we just share an elevator? Because I believe we have some serious uplifted chemistry.\n",
    "I must be a snowflake because I've fallen for you.\n",
    "Every time I see you, it feels like I've discovered a new shade of color.\n",
    "If stars would shine every time I thought of you, the night sky would never be dark.\n",
    "Do you believe in love at first sight, or should I walk by again?\n",
    "Your voice is my favorite sound, and your name is my favorite noun.\n",
    "If I were to rewrite the alphabet, I'd put U and I together.\n",
    "Have you been to the doctor recently? Because I think you've got a case of being mesmerizing.\n",
    "I wouldn't need a camera to savor this moment; I just need you.\n",
    "Is it hot in here, or is it just the aura of warmth you radiate?\n",
    "I tried finding the perfect line to make you smile, but then I realized you are the line.\n",
    "If you were a vegetable, you'd be a 'cute-cumber.'\n",
    "If life was a song, your smile would be the melody that gets stuck in my head.\n",
    "I'm not a photographer, but I can surely picture us together.\n",
    "If you were a fruit, you'd be a fine-apple.\n",
    "I'm not sure if it was the sparkle in your eyes or your radiant smile, but something just lit up my world.\n",
    "Do you mind holding my hand? I want to be sure I'm not dreaming.\n",
    "I've been trying to tie my shoes, but I can't stop tripping over my thoughts of you.\n",
    "Is there a rainbow today? Because I've just found my pot of gold in you.\n",
    "Do you have a map? I keep getting lost in your eyes.\n",
    "If you were a star, you'd outshine the entire galaxy.\n",
    "I'm not a wishful thinker, but with you, I believe in fairy tales.\n",
    "I thought happiness started with an H, but mine starts with U.\n",
    "Your smile must be a black hole because it's irresistibly attractive.\n",
    "Are you Wi-fi? Because I'm feeling a connection.\n",
    "If kisses were snowflakes, I'd send you a blizzard.\n",
    "Do you have a Band-Aid? I just scraped my knee falling for you.\n",
    "Your presence makes my heart race, but it's a marathon I never want to finish.\n",
    "If beauty were a crime, you'd be serving a life sentence.\n",
    "Did it hurt? When you fell from the stars and landed in my world?\n",
    " I'm not a genie, but I can make your dreams come true.\n",
    "Every song I hear reminds me of you, even the ones I haven't heard yet.\n",
    "I never believed in magic until I saw you.\n",
    "If you were words on a page, you'd be the fine print.\n",
    " Are you made of copper and tellurium? Because you're Cu-Te.\n",
    "Can I follow you? Because my mom told me to follow my dreams.\n",
    "Your beauty could inspire poets and artists for generations.\n",
    "Do you believe in destiny? Because I think ours just intertwined.\n",
    "Are you a calculator? Because every time I look at you, things start adding up.\n",
    "Is your name Google? Because you have everything I've been searching for.\n",
    "Do you have a name, or can I call you mine?\n",
    "Are you a magician? Every time I look at you, everyone else disappears.\n",
    "Do you like basketball? Because when I saw you, my heart did a slam dunk.\n",
    "Are you a library book? Because I can't help but check you out.\n",
    "If you were a vegetable, you'd be a cutecumber.\n",
    "Do you have a sunburn or are you always this hot?\n",
    "Is your dad a boxer? Because you're a knockout.\n",
    "Are you a loan from a bank? Because you have my interest.\n",
    "Are you a campfire? Because you're hot and I want s'more.\n",
    "Do you have a pencil? Because I want to erase your past and write our future.\n",
    "Are you a parking ticket? Because you've got 'FINE' written all over you.\n",
    "If I could rearrange the alphabet, I would put U and I together.\n",
    "Are you a snowstorm? Because you make my heart race.\n",
    "Do you have a Band-Aid? Because I just scraped my knee falling for you.\n",
    "Is your dad a baker? Because you're a cutie pie.\n",
    "Are you a time traveler? Because I can see you in my future.\n",
    "Are you a candle? Because you light up my world.\n",
    "Are you a string? Because I can't stop finding myself attached to you.\n",
    "If you were a function, I'd call you every day.\n",
    "Are you a loop? Because I can't stop thinking about you again and again.\n",
    "You're the 'key' to my 'value'.\n",
    "If beauty were data, you'd be an overflow error.\n",
    "Are you a variable? Because I feel the need to declare my intentions.\n",
    "I think you've just incremented my heart's counter.\n",
    "You're the API to my application.\n",
    "If relationships were algorithms, ours would be a perfect match.\n",
    "Your presence makes my system run at optimal efficiency.\n",
    "If you were an exception, I'd catch you.\n",
    "You must be a compiler because every time I see you, my code becomes runnable.\n",
    "I'm not a photographer, but I can picture us in an endless loop together.\n",
    "In the world of binary, you are the one for me.\n",
    "I've got a lot of lines of code, but none as intriguing as the story of us.\n",
    "Forget the cloud, you've taken up all the storage in my heart.\n",
    "I don't need version control to know you're my final commit.\n",
    "Let's merge our branches and build a beautiful codebase together.\n",
    "Do you believe in parallel universes? Because in every one I've imagined, it's always been you and me.\n",
    "You must be a packet because I can't help but sniff you out in a crowd.\n",
    "Are you an algorithm? Because every part of me wants to solve you.\n",
    "Do you know JavaScript? Because you make my heart race asynchronously.\n",
    "If I were to traverse the nodes of my heart, every path would lead to you.\n",
    "In the Git of life, I'd never want to revert our commit.\n",
    "Are you a CSS file? Because without you, my life would be unstyled.\n",
    "You must be a bug, because I just can't shake you off my mind.\n",
    "If hearts had firewalls, you'd have bypassed mine ages ago.\n",
    "Are you a Boolean? Because true or false, my heart points to you.\n",
    "In the world of objects, you are my favorite instance.\n",
    "If attraction were a programming language, ours would be high-level.\n",
    "Are you a kernel? Because my heart feels an urge to dive deeper into you.\n",
    "Do you like Java? Because you've Objectified my affections.\n",
    "I'd never put our relationship in a try-catch block, because I never want to handle an exception with us. \n",
    "You're not just a semicolon; you're the statement that completes me.\n",
    "In the IDE of life, you've just become my favorite extension.\n",
    "If I were a coder, I'd refactor my life to include more of you.\n",
    "You're like the most elegant code I've ever seen; concise, beautiful, and effective.\n",
    "You've stolen the ASCII to my heart.\n",
    "You must be an exception, because I can't handle you properly.\n",
    "You must be an algorithm, because you have my heart running in O(1) time.\n",
    "Are you a piece of code? Because you're turning my software into hardware!\n",
    "Are you a compressed file? Because when I see you, my heart wants to unzip.\n",
    "I'm the RizzLord.\n"
]

romantic_lines = [
    "Is it okay if I walk you home? My heart seems to follow wherever you go.\n",
    "If beauty were a moment, you'd be an eternity.\n",
    "Every time I see you, it feels like I've discovered a new shade of color.\n",
    "If stars would shine every time I thought of you, the night sky would glisten.\n",
    "Do you believe in love at first sight, or should I walk by again?\n",
    "Your voice is my favorite sound, and your name is my favorite noun.\n",
    "If I were to rewrite the alphabet, I'd put U and I together.\n",
    "I wouldn't need a camera to savor this moment; I just need you.\n",
    "Every song I hear reminds me of you, even the ones I haven't heard yet.\n",
    "I never believed in magic until I saw you.\n",
    "Your beauty could inspire poets and artists for generations.\n",
    "Do you believe in destiny? Because I think ours just intertwined.\n",
    "Your presence makes my heart race, but it's a marathon I never want to finish.\n",
    "If beauty were a crime, you'd be serving a life sentence.\n",
    "Every time I look at you, I believe in fairy tales.\n",
    "Thought happiness started with an H, but mine starts with U.\n",
    "Are you Wi-fi? Because I'm feeling a connection.\n",
    "Do you have a Band-Aid? I just scraped my knee falling for you.\n",
    "Is there a rainbow today? Because I've just found my pot of gold in you\n",
    "If you were a star, you'd outshine the entire galaxy.\n",
    "If kisses were snowflakes, I'd send you a blizzard.\n",
    "Your smile must be a black hole because it's irresistibly attractive.\n",
    "If you were words on a page, you'd be the fine print.\n",
    "Can I follow you? Because my mom told me to follow my dreams.\n",
    "Is your name Google? Because you have everything I've been searching for you.\n",
    "Do you have a name, or can I call you mine?\n",
    "Are you a library book? Because I can't help but check you out.\n",
    "Do you have a pencil? Because I want to erase your past and write our future.\n",
    "Is your dad a baker? Because you're a cutie pie.\n",
    "Are you a candle? Because you light up my world.\n",
    "Are you a string? Because I can't stop finding myself attached to you.\n",
    "If you were a function, I'd call you every day.\n",
    "Are you a loop? Because I can't stop thinking about you again and again.\n"
]

clever_lines = [
    "Did we just share an elevator? Because I believe we have some serious uplifted chemistry.\n",
    "I must be a snowflake because I've fallen for you.\n",
    "I tried finding the perfect line to make you smile, but then I realized you are the line.\n",
    "If you were a vegetable, you'd be a 'cute-cumber.\n",
    "If life was a song, your smile would be the melody that gets stuck in my head.\n",
    "Is it hot in here, or is it just the aura of warmth you radiate?\n",
    "I've been trying to tie my shoes, but I can't stop tripping over my thoughts of you.\n",
    "Do you have a map? I keep getting lost in your eyes.\n",
    "Did it hurt? When you fell from the stars and landed in my world?\n",
    "I'm not a genie, but I can make your dreams come true.\n",
    "Are you made of copper and tellurium? Because you're Cu-Te.\n",
    "Are you a magician? Every time I look at you, everyone else disappears.\n",
    "Do you like basketball? Because when I saw you, my heart did a slam dunk.\n",
    "If you were a vegetable, you'd be a cutecumber.\n",
    "Do you have a sunburn, or are you always this hot?\n",
    "Is your dad a boxer? Because you're a knockout.\n",
    "Are you a loan from a bank? Because you have my interest.\n",
    "Are you a campfire? Because you're hot and I want s'more.\n",
    "Are you a parking ticket? Because you've got 'FINE' written all over you.\n",
    "Are you a snowstorm? Because you make my heart race.\n",
    "Are you a time traveler? Because I can see you in my future.\n"
]

geeky_lines = [
    "You're the 'key' to my 'value'.\n",
    "If beauty were data, you'd be an overflow error.\n",
    "Are you a variable? Because I feel the need to declare my intentions.\n",
    "I think you've just incremented my heart's counter.\n",
    "You're the API to my application.\n",
    "If relationships were algorithms, ours would be a perfect match.\n",
    "Your presence makes my system run at optimal efficiency.\n",
    "If you were an exception, I'd catch you.\n",
    "You must be a compiler because every time I see you, my code becomes runnable.\n",
    "I'm not a photographer, but I can picture us in an endless loop together.\n",
    "In the world of binary, you are the one for me.\n",
    "I've got a lot of lines of code, but none as intriguing as the story of us.\n",
    "Forget the cloud, you've taken up all the storage in my heart.\n",
    "I don't need version control to know you're my final commit.\n",
    "Let's merge our branches and build a beautiful codebase together.\n",
    "Do you believe in parallel universes? Because in every one I've imagined, it's always been you and me.\n",
    "You must be a packet because I can't help but sniff you out in a crowd.\n",
    "Are you an algorithm? Because every part of me wants to solve you.\n",
    "Do you know JavaScript? Because you make my heart race asynchronously.\n",
    "If I were to traverse the nodes of my heart, every path would lead to you.\n",
    "In the Git of life, I'd never want to revert our commit.\n",
    "Are you a CSS file? Because without you, my life would be unstyled.\n",
    "You must be a bug, because I just can't shake you off my mind.\n",
    "If hearts had firewalls, you'd have bypassed mine ages ago.\n",
    "Are you a Boolean? Because true or false, my heart points to you.\n",
    "In the world of objects, you are my favorite instance.\n",
    "If attraction were a programming language, ours would be high-level.\n",
    "Are you a kernel? Because my heart feels an urge to dive deeper into you.\n",
    "Do you like Java? Because you've Objectified my affections.\n",
    "I'd never put our relationship in a try-catch block, because I never want to handle an exception with us.\n",
    "You're not just a semicolon; you're the statement that completes me.\n",
    "In the IDE of life, you've just become my favorite extension.\n",
    "If I were a coder, I'd refactor my life to include more of you.\n",
    "You're like the most elegant code I've ever seen; concise, beautiful, and effective.\n",
    "You have stolen the ASCII to my heart.\n",
    "You must be an exception, because I can't handle you properly.\n",
    "You must be an algorithm, because you have my heart running in O(1) time.\n",
    "Are you a piece of code? Because you're turning my software into hardware!\n",
    "Are you a compressed file? Because when I see you, my heart wants to unzip.\n"
]

pickuplines = {
    'dev': dev_lines,
    'romantic': romantic_lines,
    'clever': clever_lines,
    'geeky': geeky_lines,
    'all': dev_lines + romantic_lines + clever_lines + geeky_lines
}


# Uncomment when using pytest and uploading the package to PyPi
import pyrizz.pyrizz as pyrizz

# Uncomment when testing the __main__.py file locally
# import pyrizz as pyrizz

"""Main function for PyRizz."""

#ASCII art

print(" ________  ___    ___ ________  ___  ________  ________        ")
print("|\   __  \|\  \  /  /|\   __  \|\  \|\_____  \|\_____  \       ")
print("\ \  \|\  \ \  \/  / | \  \|\  \ \  \\\\|___/  /|\|___/  /|      ")
print(" \ \   ____\ \    / / \ \   _  _\ \  \   /  / /    /  / /      ")
print("  \ \  \___|\/   / /   \ \  \\\\  \\\\ \  \ /  /_/__  /  /_/__     ")
print("   \ \__\ __/   / /     \ \__\\\\ _\\\\ \__\\\\________\\\\________\   ")
print("    \|__||\____/ /       \|__|\|__|\|__|\|_______|\|_______|   ")
print("         \|____|/                                               ")

print("\n")

def main():
    openai_client = None
    print("Welcome to PyRizz! Your journey to getting a date begins here...")

    while True: 
        print("\nWhat would you like to do today?\n")
        print("1. Get a random pick-up line")
        print("2. Get a category-specific random pick-up line (romantic, clever, geeky, dev)")
        print("3. Create your line with one of our templates with randomly selected ASCII art.")
        print("4. List available templates for pick-up lines.\n")
        print("Use AI for your pickup lines: ")
        print("5. Enter your API key to use AI functionality.")
        print("6. Have AI generate a pick-up line in your chosen category / language (no more than 50 characters)")
        print("7. Have AI rate your pick-up line out of 10. Test it on AI before trying it on a human! ;) \n")
        
        print("!! Type Q to quit !!\n")

        print("Enter your choice: ")
        user_input = input("> ")

        if user_input == "1":
            print("\n" + pyrizz.get_random_line(), end = "\n")

        elif user_input == "2":
            print("Select a category: ")
            print("1 - Romantic")
            print("2 - Clever")
            print("3 - Geeky")
            print("4 - Developer Lines")
            category_index = input("> ")
            if (category_index.isdigit()): 
                category_index = int(category_index) 
                if(category_index in range(1, 5)):
                    if(category_index == 1): 
                        category_val = "romantic"
                    elif(category_index == 2): 
                        category_val = "clever"
                    elif(category_index == 3):
                        category_val = "geeky"
                    elif(category_index == 4):
                        category_val = "dev"
                    print("\n" + pyrizz.get_random_category_line(category_val), end = "\n")
                else: 
                    print("Please make sure it is a number from 1-4. \n")
            else: 
                print("Not a valid input! \n")

        elif user_input == "3": 
            template_number, words = pyrizz.get_user_input_for_line()
            line = pyrizz.create_line(template_number, words)
            if line:
                print("\nHere's your custom pick-up line:")
                print(line + "\n")

        elif user_input == "4":
            print("Here are the available templates:")
            templates = pyrizz.list_templates()
            for idx, template in enumerate(templates, 1):
                print(f"Template {idx}: {template}")

        elif user_input == "5":
            print("Please enter your API key.")
            user_api_key = input("> ")
            openai_client = pyrizz.init_openai(user_api_key)
        
        elif user_input == "6":
            if openai_client:
                print("Enter a category / language: ")
                category = input("> ")
                print("\n" + pyrizz.get_ai_line(category, openai_client), end = "\n\n")
            else:
                print("\nYou need to enter an API key first. Select 5!\n")
        
        elif user_input == "7":
            if openai_client:
                print("Type your pickup line: ")
                pickup_line = input("> ")
                print("\n" + pyrizz.rate_line(pickup_line, openai_client), end = "\n\n")
            else:
                print("\nYou need to enter an API key first. Select 5!\n")

        elif user_input == "q" or user_input == "Q":
            break
        
        else:
            print("\nInvalid Response.")
        
        while True:
            user_cont = input("Would you like to continue? (y/n): ")
            if user_cont in ["y", "n"]:
                break
            else:
                print("\nPlease provide a valid input (y/n)")

        if user_cont == "n" or user_cont == "q" or user_cont == "Q":
            break
        
if __name__ == "__main__":
    main()


import pytest
import pathlib
import sys
import re
import unittest
from unittest.mock import Mock, patch
from unittest.mock import patch, MagicMock, mock_open
import openai
sys.path.append(f"{pathlib.Path(__file__).parent.resolve()}/../src")
from pyrizz import pyrizz

class Tests:

    # Tests if this returns a list
    def test_get_lines_list(self):
        actual = pyrizz.get_lines('all')
        assert isinstance(actual, list)
    
    # Tests if this returns an empty or not a proper category
    def test_get_lines_nonempty(self):
        actual = pyrizz.get_lines()
        assert isinstance(actual, list)
        assert bool(actual) is True


    # Tests if this returns a string
    def test_get_random_line_str(self):
        actual = pyrizz.get_random_line()
        assert isinstance(actual, str)

    # Tests if the get random line is a non-empty value
    def test_get_random_line_nonempty(self):
        actual = pyrizz.get_random_line()
        assert isinstance(actual, str)
        assert len(actual) > 0

    # Tests if the get random line is a empty value
    def test_get_random_line_empty(self):
        actual = pyrizz.get_random_line()
        assert isinstance(actual, str)
        assert bool(actual)

    # Tests if the get random line is long enough
    def test_get_random_line_long_enough(self):
        actual = pyrizz.get_random_line()
        assert isinstance(actual, str)
        assert len(actual) > 1


    # Tests if this returns a string
    def test_get_random_category_line_str(self):
        actual = pyrizz.get_random_category_line("romantic")
        assert isinstance(actual, str)

    # Tests if the get random category line is a non-empty value
    def test_get_random_category_line_nonempty(self):
        actual = pyrizz.get_random_category_line("romantic")
        assert isinstance(actual, str)
        assert len(actual) > 0

    # Tests if the get random cateogory line is a empty value
    def test_get_random_category_line_empty(self):
        actual = pyrizz.get_random_category_line()
        assert isinstance(actual, str)
        assert bool(actual)

    # Tests if the get random line is long enough
    def test_get_random_category_line_longenough(self):
        actual = pyrizz.get_random_category_line("romantic")
        assert isinstance(actual, str)
        assert len(actual) > 0
    
    # Note: Not testing init_openai(key) because there is not any logic; just working on openai API functionality, authentication, and connection."""
    # Tests if the input for ai line is empty
    def test_get_ai_line_empty(self):
        helper = Helper
        mock_client = helper.create_get_mock()
        actual = pyrizz.get_ai_line("", mock_client)
        expected = "Please specify a category."
        assert actual.strip() == expected.strip()

    # Tests if the input is way too long
    def test_get_ai_line_long(self):
        helper = Helper
        mock_client = helper.create_get_mock()
        actual = pyrizz.get_ai_line("This is a very long category that is definitely more than 50 characters long.", mock_client)
        expected = "Please specify a category that is less than 50 characters."
        assert actual.strip() == expected.strip()

    # Tests if the input for ai line actually results in a string
    def test_get_ai_line_str(self):
        helper = Helper
        mock_client = helper.create_get_mock()
        actual = pyrizz.get_ai_line("test", mock_client)
        assert isinstance(actual, str)

    # Tests if the rate line is empty
    def test_rate_line_empty(self):
        helper = Helper
        mock_client = helper.create_rate_mock()
        actual = pyrizz.rate_line("", mock_client)  
        assert actual == "No pickup line? You gotta use our other features before you come here buddy."

    # Tests if the rate line function follows a specific format
    def test_rate_line_format(self):
        helper = Helper
        mock_client = helper.create_rate_mock()
        actual = pyrizz.rate_line("Do you come with Wi-Fi? Because I'm really feeling a connection.", mock_client)
        assert re.match(r'\d+/10 - .+', actual) is not None

    #Tests if the rate line function returns 
    def test_rate_line_gibberish(self):
        helper = Helper
        mock_client = helper.create_rate_mock()
        actual = pyrizz.rate_line("jwrkf", mock_client)
        assert re.match(r'.+', actual) is not None
     
    # Tests for user input validation 
    def test_is_line_valid_length(self):
        long_line = "x" * 141  
        assert not pyrizz.is_line_valid(long_line), "Expected the line to be flagged as too long."
    
    # Test for invalid tempalte number
    def test_create_line_invalid_template_number(self):
        _, message = pyrizz.create_line(999, ["word1", "word2"])
        assert message == "Template number out of range. Please choose between 0 and {}.".format(len(pyrizz.templates) - 1)
    
    # Test for incorrect word count 
    def test_create_line_incorrect_word_count(self):
        templates = ["Template with one placeholder: {}"]
        with patch('pyrizz.templates', new=templates):
            _, message = pyrizz.create_line(0, ["word1", "word2"])
            expected_message = "Incorrect number of words provided for the placeholders. Expected 1, got 2."
            assert message == expected_message
    
    # Test for non-integer template number
    @patch('builtins.input', side_effect=["not_an_integer"])
    def test_get_user_input_for_line_noninteger(self, mock_input):
        with patch('pyrizz.templates', new=["Some template"]):
            template_number, words = pyrizz.get_user_input_for_line()
            assert template_number is None and words is None
    
    # Test for out of range template numebr
    @patch('builtins.input', side_effect=["99", "word1, word2"])
    def test_get_user_input_for_line_out_of_range(self, mock_input):
        templates = ["Template 0"]
        with patch('pyrizz.templates', new=templates):
            template_number, words = pyrizz.get_user_input_for_line()
            assert template_number is None and words is None
    
    # Test for incorrect number of words provided
    @patch('builtins.input', side_effect=["0", "word1, word2, word3"])
    def test_get_user_input_for_line_incorrect_word_count(self, mock_input):
        templates = ["Template with two placeholders: {}, {}"]
        with patch('pyrizz.templates', new=templates):
            template_number, words = pyrizz.get_user_input_for_line()
            assert template_number is None and words is None


class Helper:
    def create_get_mock():
        mock_client = MagicMock()

        mock_response = {
            "choices": [
                {
                    "message": {
                        "content": "Test content"
                    }
                }
            ]
        }
        mock_client.ChatCompletion.create.return_value = MagicMock(**mock_response)

        return mock_client
    
    def create_rate_mock():
        mock_client = MagicMock()

        mock_response = {
            "choices": [
                {
                    "message": {
                        "content": "10/10 - Short, sweet, and guaranteed to make anyone blush!"
                    }
                }
            ]
        }
        mock_client.ChatCompletion.create.return_value = MagicMock(**mock_response)

        return mock_client
